#include <QtWidgets/QApplication> // for Qt functions
#include "Viewer.h" // PoLAR::Viewer header
#include "Image.h"
#include "Util.h"

#include <iostream>
#include <QApplication>
#include <osg/Geode>
#include <osg/Geometry>
#include <osgDB/ReadFile>
#include <osgUtil/SmoothingVisitor>
#include <osgUtil/Optimizer>
#include <osg/io_utils>

int main(int argc, char** argv)
{
    // Create the Qt application
    QApplication app(argc, argv);
    PoLAR::Viewer viewer(0, "Viewer", 0, true);
    osg::ref_ptr<PoLAR::Image<unsigned char> > myImage;

    // Load background image
    myImage = new PoLAR::Image<unsigned char>("../../salon.jpg", true, 1);
    viewer.setBgImage(myImage);
    viewer.bgImageOn();

    // Resize the widget
    int width = myImage->getWidth();
    int height = myImage->getHeight();
    viewer.resize(width, height);
    osgUtil::SmoothingVisitor smooth;
    osgUtil::Optimizer optimizer;


    //chargement de la chaise
    osg::ref_ptr<osg::Node> loadedModel2;
    loadedModel2 = osgDB::readNodeFile("../../squeezer.obj");

    optimizer.optimize(loadedModel2.get());
    loadedModel2->accept(smooth);
    // Add the object3D created from loaded model to the viewer
    osg::ref_ptr<PoLAR::Object3D> obj2 = new PoLAR::Object3D(loadedModel2.get(), true, true);
    obj2->scale(2.0f);
    obj2->translate(10,10,-2);
    
    obj2->rotate(M_PI, 0, 1, 0);

    osg::ref_ptr<osg::Node> loadedModel1;
    loadedModel1 = osgDB::readNodeFile("../../broom.obj");

    optimizer.optimize(loadedModel1.get());
    loadedModel1->accept(smooth);

    osg::ref_ptr<PoLAR::Object3D> obj1 = new PoLAR::Object3D(loadedModel1.get(), true, true);
    obj1->scale(30.0f);
    obj1->translate(10,-10,0);
    obj1->rotate(M_PI, 0, 1, 0);

    PoLAR::Viewer::ProjectionType pt=PoLAR::Viewer2D3D::VISION;
    osg::Matrixd P = PoLAR::Util::readProjectionMatrix("../../matrix.txt");
    viewer.setProjection(P, pt);

    osg::ref_ptr<osg::Geode> tableGeode = new osg::Geode;
    osg::ref_ptr<osg::ShapeDrawable> shape;
    shape = new osg::ShapeDrawable(new osg::Box(osg::Vec3(25.0f, 12.5f, 0.0f), 50.0f, 25.0f, 0.00001f));
    tableGeode->addDrawable(shape.get());
    osg::ref_ptr<PoLAR::Object3D> object3D = new PoLAR::Object3D(tableGeode.get());
    object3D->setName("Table");
    object3D->setPhantomOn();
     viewer.addObject3D(object3D.get());
     
    viewer.addLightSource(20,15, -9, true);
    viewer.addObject3D(obj2.get());
    viewer.setTrackNode(obj2.get());
    viewer.addObject3D(obj1.get());
    viewer.setTrackNode(obj1.get());

    // Show the widget
    viewer.show();
    // This line ensures the application will properly close when the widget has been closed
    app.connect( &app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()) );
    // Run the app
    return app.exec();
}	